# Configurations

# (0xB0) Configure LoRa parameters
lora_params = False
lora_adr = True                 # ADR enable/disable
lora_dr = 2                     # 0... 5
lora_send_trials = 3            # 1... 8
lora_join_trials = 1            # 1... 9
lora_port = 1                   # 1... 223
lora_cnf = True                 # confirmed messages enable/disable
lora_heartbeat = 48             # 0... 65535‬
lora_interval = 7               # 5... 65535‬

# (0xB6) Configure periphery
periphery_params = False
periphery_app_mode = 1          # 1 (send on all buttons), 2 (send on button 6), 3 (send on button 1)
periphery_piezo_volume = 2      # 0 (Off)... 3 (loudest)
periphery_piezo_mode = 0        # 0 (1 tone), 1 (2 tones), 2 (2 tones 2*dur1), 3 (2 tones 2*dur2)
periphery_piezo_freq2 = 2       # 0 (500Hz), 1 (1kHz), 2 (2kHz), 3 (4kHz)
periphery_piezo_freq1 = 1       # 0 (500Hz), 1 (1kHz), 2 (2kHz), 3 (4kHz)
periphery_vibra_enable = True
periphery_accel_mode = 0        # 0... 6
periphery_buttons_enable = 0x3F # button EN bits: 0b0011'1111
periphery_button_press_dur_short = 1000   # 0... 8000ms
periphery_button_press_dur_long = 5000    # 0... 8000ms

# (0xB7) Select colors
color_params = False
color_success = [0, 0x8F, 0]    # rgb
color_failure = [0xFF, 0, 0]    # rgb
color_loading = [0xFF, 0, 0xFF] # rgb
color_ble =     [0, 0, 0xFF]    # rgb

# (0xB8) Send Feedback after Uplink
feedback_params = False
feedback_state = 1              # 1 => Positiv Feedback (default green) / 0 = negative Feedback (default red)

# (0xC0) Get Power Saving Settings
get_power_params = False
            
# (0xC1) Set Power Saving Settings
set_power_params = False
power_batch_time = 0            # 0 seconds => batchmode off / Minumum 600 Seconds
power_led_power_save_mode = 0   # 0 => normal Led Mode / 1 => Save energy with optimized Led animations
power_send_info = 3             # 0 Sending Process and Feedback are not shown with LED.
                                # 1 Only Transmission Feedback is shown.
                                # 2 Only Sending Process is shown.
                                # 3 Sending Process and Feedback are shown with LED.
                                
                                

#########################################################################
# (0xB0) Configure LoRa parameters
print("--> Button Board LoRa Downlink generator <--\n")
downlink_message = ""
if lora_params:
    print("generating the lora_params...")
    downlink_message += "B0"
    downlink_message += "01" if lora_adr else "00"
    if lora_dr < 0 or lora_dr > 5: print("-> invalid LoRa DR!")
    downlink_message += ''.join(format(lora_dr, '02X'))
    if lora_send_trials < 1 or lora_send_trials > 8: print("-> invalid LoRa send trials!")
    downlink_message += ''.join(format(lora_send_trials, '02X'))
    if lora_join_trials < 1 or lora_join_trials > 9: print("-> invalid LoRa join trials!")
    downlink_message += ''.join(format(lora_join_trials, '02X'))
    if lora_port < 1 or lora_port > 223: print("-> invalid LoRa port!")
    downlink_message += ''.join(format(lora_port, '02X'))
    downlink_message += "01" if lora_cnf else "00"
    if lora_heartbeat < 0 or lora_heartbeat > 65535: print("-> invalid LoRa heartbeat!")
    downlink_message += ''.join(format(lora_heartbeat, '04X'))
    if lora_interval < 5 or lora_interval > 65535: print("-> invalid LoRa interval!")
    downlink_message += ''.join(format(lora_interval, '04X'))

# (0xB6) Configure periphery
if periphery_params:
    print("generating the periphery_params...")
    downlink_message += "B6"
    if periphery_app_mode < 1 or periphery_app_mode > 3:  print("-> invalid app mode!")
    downlink_message += ''.join(format(periphery_app_mode, '02X'))
    if periphery_piezo_mode < 0 or periphery_piezo_mode > 3:  print("-> invalid piezo mode!")
    if periphery_piezo_volume < 0 or periphery_piezo_volume > 3:  print("-> invalid piezo volume!")
    periphery_piezo_freq = (periphery_piezo_volume << 2) | periphery_piezo_mode
    downlink_message += ''.join(format(periphery_piezo_freq, '01X'))
    if periphery_piezo_freq1 < 0 or periphery_piezo_freq1 > 3:  print("-> invalid piezo frequency1!")
    if periphery_piezo_freq2 < 0 or periphery_piezo_freq2 > 3:  print("-> invalid piezo frequency2!")
    periphery_piezo_freq = (periphery_piezo_freq2 << 2) | periphery_piezo_freq1
    downlink_message += ''.join(format(periphery_piezo_freq, '01X'))
    downlink_message += "01" if periphery_vibra_enable else "00"
    if periphery_accel_mode < 0 or periphery_accel_mode > 6:  print("-> invalid accel mode!")
    downlink_message += ''.join(format(periphery_accel_mode, '02X'))
    if periphery_buttons_enable <= 0 or periphery_buttons_enable > 0x3F:  print("-> invalid buttons enable!")
    downlink_message += ''.join(format(periphery_buttons_enable, '02X'))
    if periphery_button_press_dur_short < 0 or periphery_button_press_dur_short > 8000: print("-> invalid button press dur short!")
    downlink_message += ''.join(format(periphery_button_press_dur_short, '04X'))
    if periphery_button_press_dur_long < 0 or periphery_button_press_dur_long > 8000: print("-> invalid button press dur long!")
    downlink_message += ''.join(format(periphery_button_press_dur_long, '04X'))

# (0xB7) Select colors
if color_params:
    print("generating the color_params...")
    downlink_message += "B7"
    for color in color_success:
        downlink_message += ''.join(format(color, '02X'))
    for color in color_failure:
        downlink_message += ''.join(format(color, '02X'))
    for color in color_loading:
        downlink_message += ''.join(format(color, '02X'))
    for color in color_ble:
        downlink_message += ''.join(format(color, '02X'))
        
# (0xB8) Send Feedback after Uplink        
if feedback_params:
    print("generating the feedback massage...")
    downlink_message += "B8"
    downlink_message += ''.join(format(feedback_state, '02X'))

# (0xC0) Get Power Saving Settings       
if get_power_params:
    print("generating message to get power params...")
    downlink_message += "C0"
    
# (0xC1) Set Power Saving Settings      
if set_power_params:
    print("generating message to get power params...")
    downlink_message += "C1"
    downlink_message += ''.join(format(power_batch_time, '04X'))
    downlink_message += ''.join(format(power_led_power_save_mode, '02X'))
    downlink_message += ''.join(format(power_send_info, '02X'))

# Print Lora Downlink
print("\nFinal LoRa downlink: ==> " + downlink_message + " <== (" + str(len(downlink_message)//2) + "/50 bytes)")
if len(downlink_message)//2 > 50: print("-> Error: Too many bytes!")
print("")
